@@ -2,6 +2,7 @@
 
 import json
 import os
+from pathlib import Path
 from tempfile import NamedTemporaryFile
 from typing import Dict, List, Optional, TextIO
 
@@ -11,6 +12,7 @@ import mypy.api
 import pytest  # type: ignore
 
 
+PYTEST_MAJOR_VERSION = int(pytest.__version__.partition(".")[0])
 mypy_argv = []
 nodeid_name = "mypy"
 
@@ -32,6 +34,12 @@ def pytest_addoption(parser):
         action="store_true",
         help="suppresses error messages about imports that cannot be resolved",
     )
+    group.addoption(
+        "--mypy-config-file",
+        action="store",
+        type=str,
+        help="adds custom mypy config file",
+    )
 
 
 XDIST_WORKERINPUT_ATTRIBUTE_NAMES = (
@@ -87,10 +95,6 @@ def pytest_configure(config):
 
             config.pluginmanager.register(_MypyXdistPlugin())
 
-    # pytest_terminal_summary cannot accept config before pytest 4.2.
-    global _pytest_terminal_summary_config
-    _pytest_terminal_summary_config = config
-
     config.addinivalue_line(
         "markers",
         "{marker}: mark tests to be checked by mypy.".format(marker=MypyItem.MARKER),
@@ -98,20 +102,40 @@ def pytest_configure(config):
     if config.getoption("--mypy-ignore-missing-imports"):
         mypy_argv.append("--ignore-missing-imports")
 
+    mypy_config_file = config.getoption("--mypy-config-file")
+    if mypy_config_file:
+        mypy_argv.append("--config-file={}".format(mypy_config_file))
 
-def pytest_collect_file(path, parent):
+
+def pytest_collect_file(file_path, parent):
     """Create a MypyFileItem for every file mypy should run on."""
-    if path.ext in {".py", ".pyi"} and any(
-        [parent.config.option.mypy, parent.config.option.mypy_ignore_missing_imports],
+    if file_path.suffix in {".py", ".pyi"} and any(
+        [
+            parent.config.option.mypy,
+            parent.config.option.mypy_config_file,
+            parent.config.option.mypy_ignore_missing_imports,
+        ],
     ):
         # Do not create MypyFile instance for a .py file if a
         # .pyi file with the same name already exists;
         # pytest will complain about duplicate modules otherwise
-        if path.ext == ".pyi" or not path.new(ext=".pyi").isfile():
-            return MypyFile.from_parent(parent=parent, fspath=path)
+        if file_path.suffix == ".pyi" or not file_path.with_suffix(".pyi").is_file():
+            return MypyFile.from_parent(parent=parent, path=file_path)
     return None
 
 
+if PYTEST_MAJOR_VERSION < 7:  # pragma: no cover
+    _pytest_collect_file = pytest_collect_file
+
+    def pytest_collect_file(path, parent):  # type: ignore
+        try:
+            # https://docs.pytest.org/en/7.0.x/deprecations.html#py-path-local-arguments-for-hooks-replaced-with-pathlib-path
+            return _pytest_collect_file(Path(str(path)), parent)
+        except TypeError:
+            # https://docs.pytest.org/en/7.0.x/deprecations.html#fspath-argument-for-node-constructors-replaced-with-pathlib-path
+            return MypyFile.from_parent(parent=parent, fspath=path)
+
+
 class MypyFile(pytest.File):
 
     """A File that Mypy will run on."""
@@ -292,9 +316,8 @@ class MypyError(Exception):
     """
 
 
-def pytest_terminal_summary(terminalreporter):
+def pytest_terminal_summary(terminalreporter, config):
     """Report stderr and unrecognized lines from stdout."""
-    config = _pytest_terminal_summary_config
     try:
         with open(config._mypy_results_path, mode="r") as results_f:
             results = MypyResults.load(results_f)
